"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('gatsby-source-filesystem'),
    createRemoteFileNode = _require.createRemoteFileNode;

var gcloudCmd = require("./gcloud-cmd");

var GCloudObjectType = "GCloudObject";

var GCloudObject = function GCloudObject(baseUrl, _ref) {
    var bucket = _ref.bucket,
        name = _ref.name,
        contentType = _ref.contentType,
        timeCreated = _ref.timeCreated,
        updated = _ref.updated,
        md5Hash = _ref.md5Hash,
        etag = _ref.etag;
    return {
        baseUrl: baseUrl,
        bucket: bucket,
        name: name,
        contentType: contentType,
        timeCreated: timeCreated,
        updated: updated,
        md5Hash: md5Hash,
        etag: etag
    };
};

var getObjectURL = function getObjectURL(o) {
    return o.baseUrl + "/" + o.bucket + "/" + o.name;
};

var createRemoteGCloudNode = function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref2) {
        var url = _ref2.url,
            store = _ref2.store,
            cache = _ref2.cache,
            createNode = _ref2.createNode;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        _context.prev = 0;
                        _context.next = 3;
                        return createRemoteFileNode({
                            url: url,
                            store: store,
                            cache: cache,
                            createNode: createNode
                        });

                    case 3:
                        return _context.abrupt("return", _context.sent);

                    case 6:
                        _context.prev = 6;
                        _context.t0 = _context["catch"](0);

                        console.err("Unable to create remote node for: " + url);
                        return _context.abrupt("return", null);

                    case 10:
                    case "end":
                        return _context.stop();
                }
            }
        }, _callee, undefined, [[0, 6]]);
    }));

    return function createRemoteGCloudNode(_x) {
        return _ref3.apply(this, arguments);
    };
}();

var createGCloudObjectNode = function () {
    var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(_ref4) {
        var createNode = _ref4.createNode,
            fileNode = _ref4.fileNode,
            objectData = _ref4.objectData;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        createNode(_extends({}, objectData, {

                            //gatsby data
                            id: objectData.bucket + "/" + objectData.name + " >> " + GCloudObjectType,
                            absolutePath: fileNode.absolutePath,
                            parent: fileNode.id,
                            children: [],
                            internal: {
                                content: getObjectURL(objectData),
                                contentDigest: objectData.md5Hash,
                                mediaType: objectData.contentType,
                                type: GCloudObjectType
                            }
                        }));

                    case 1:
                    case "end":
                        return _context2.stop();
                }
            }
        }, _callee2, undefined);
    }));

    return function createGCloudObjectNode(_x2) {
        return _ref5.apply(this, arguments);
    };
}();

exports.sourceNodes = function () {
    var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref6, //gatsby elements
    _ref7, //plugin options
    done //completion callback
    ) {
        var boundActionCreators = _ref6.boundActionCreators,
            getNode = _ref6.getNode,
            hasNodeChanged = _ref6.hasNodeChanged,
            store = _ref6.store,
            cache = _ref6.cache;
        var project = _ref7.project,
            bucket = _ref7.bucket,
            _ref7$prefix = _ref7.prefix,
            prefix = _ref7$prefix === undefined ? "" : _ref7$prefix,
            _ref7$domain = _ref7.domain,
            domain = _ref7$domain === undefined ? "storage.googleapis.com" : _ref7$domain,
            _ref7$protocol = _ref7.protocol,
            protocol = _ref7$protocol === undefined ? "https" : _ref7$protocol;
        var createNode, objects;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        createNode = boundActionCreators.createNode;

                        //let's get all the files in the target bucket and let's create a node GCloudObject
                        //linked to a RemoteFileNode

                        _context4.next = 3;
                        return gcloudCmd.getAllObjects(project, bucket, prefix).map(function (o) {
                            return GCloudObject(protocol + "://" + domain, o.metadata);
                        });

                    case 3:
                        objects = _context4.sent;
                        _context4.next = 6;
                        return Promise.all(objects.map(function () {
                            var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(objectData) {
                                var fileNode;
                                return regeneratorRuntime.wrap(function _callee3$(_context3) {
                                    while (1) {
                                        switch (_context3.prev = _context3.next) {
                                            case 0:
                                                _context3.next = 2;
                                                return createRemoteGCloudNode({
                                                    url: getObjectURL(objectData),
                                                    store: store,
                                                    cache: cache,
                                                    createNode: createNode
                                                });

                                            case 2:
                                                fileNode = _context3.sent;

                                                if (fileNode) {
                                                    _context3.next = 5;
                                                    break;
                                                }

                                                return _context3.abrupt("return");

                                            case 5:
                                                //create the GCloudObject node
                                                //Remember to link it to the remote file node with localFile___NODE
                                                objectData.localFile___NODE = fileNode.id;
                                                _context3.next = 8;
                                                return createGCloudObjectNode({
                                                    createNode: createNode,
                                                    fileNode: fileNode,
                                                    objectData: objectData
                                                });

                                            case 8:
                                            case "end":
                                                return _context3.stop();
                                        }
                                    }
                                }, _callee3, undefined);
                            }));

                            return function (_x6) {
                                return _ref9.apply(this, arguments);
                            };
                        }()));

                    case 6:

                        done();

                    case 7:
                    case "end":
                        return _context4.stop();
                }
            }
        }, _callee4, undefined);
    }));

    return function (_x3, _x4, _x5) {
        return _ref8.apply(this, arguments);
    };
}();